/*
    geccouserman -- A user and group management plug-in
    Copyright (C) 1999  <John Vivian>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <gnome.h>

#include "geccouserman.h"
#include "main.h"

#include <stdio.h>
#include <malloc.h>


void *gecco_plugin_init ( int );
int gecco_plugin_run ( GtkWidget * );


static geccouserman_t *geccouserman_new ( GtkWidget * );
static int main_button_press ( GtkWidget *, gint, geccouserman_t * );
void help_button ( geccouserman_t * );
static int gusermanshutdown ( GtkWidget *, geccouserman_t * );



static geccouserman_t	*guserman = NULL;




static const char	*GECCOUSERMAN_AUTHORS[] = { "Scott Fritzinger",
						"John Vivian",
						NULL };



/*
 * Define list of buttons to have in dialog box
 */
static char	*dbuttons[] = {
			GNOME_STOCK_BUTTON_OK,
			GNOME_STOCK_BUTTON_APPLY,
			GNOME_STOCK_BUTTON_CANCEL,
			GNOME_STOCK_BUTTON_HELP,
			NULL
			};



/* XPM */
/* XPM */
static char * geccoicon_xpm[] = {
"32 32 138 2",
"  	c None",
". 	c #8FB47F",
"+ 	c #C7D9BF",
"@ 	c #59943F",
"# 	c #5A973F",
"$ 	c #CDEFBF",
"% 	c #93C17F",
"& 	c #578F3F",
"* 	c #206A00",
"= 	c #549B35",
"- 	c #9CDF7F",
"; 	c #E4FFD8",
"> 	c #216F00",
", 	c #4D9C2A",
"' 	c #CCEABF",
") 	c #95C87F",
"! 	c #5B9C3F",
"~ 	c #C1FDA6",
"{ 	c #216E00",
"] 	c #216D00",
"^ 	c #5DA03F",
"/ 	c #58903F",
"( 	c #58913F",
"_ 	c #90B67F",
": 	c #5A983F",
"< 	c #41A814",
"[ 	c #F0FFEA",
"} 	c #C1E1B3",
"| 	c #206B00",
"1 	c #2D9600",
"2 	c #42B410",
"3 	c #84CE64",
"4 	c #DFFFD1",
"5 	c #C9E0BF",
"6 	c #69BF44",
"7 	c #DFFFD2",
"8 	c #C9E1BF",
"9 	c #2A8B00",
"0 	c #2D9002",
"a 	c #B4EA9D",
"b 	c #95C77F",
"c 	c #5DA23F",
"d 	c #288400",
"e 	c #278200",
"f 	c #2D9400",
"g 	c #237600",
"h 	c #B5EA9D",
"i 	c #C1D9B6",
"j 	c #99D47F",
"k 	c #257D00",
"l 	c #319903",
"m 	c #278300",
"n 	c #92BE7F",
"o 	c #2B8E00",
"p 	c #FAFFF9",
"q 	c #AAEC8D",
"r 	c #B9EFA1",
"s 	c #36960C",
"t 	c #A0DF85",
"u 	c #217000",
"v 	c #EBFFE2",
"w 	c #3DAB0E",
"x 	c #4AB41C",
"y 	c #49B41A",
"z 	c #FCFFFB",
"A 	c #B2DF9E",
"B 	c #4AA622",
"C 	c #2C8F01",
"D 	c #268000",
"E 	c #4F8F33",
"F 	c #298800",
"G 	c #7AB461",
"H 	c #59B432",
"I 	c #4CB41F",
"J 	c #B2D9A1",
"K 	c #B3E09F",
"L 	c #2A8C00",
"M 	c #99D37F",
"N 	c #298A00",
"O 	c #EFFFE8",
"P 	c #89D468",
"Q 	c #237500",
"R 	c #5EA53F",
"S 	c #568F3D",
"T 	c #3EB40B",
"U 	c #39B204",
"V 	c #267F00",
"W 	c #4CA823",
"X 	c #489526",
"Y 	c #539F32",
"Z 	c #6DB74E",
"` 	c #257B00",
" .	c #2F9E00",
"..	c #78D94D",
"+.	c #7FB468",
"@.	c #93C27F",
"#.	c #40AA11",
"$.	c #99D57F",
"%.	c #2D8F03",
"&.	c #ECFFE4",
"*.	c #4DA327",
"=.	c #7EB467",
"-.	c #ECFFE5",
";.	c #8DCC72",
">.	c #BAFF9C",
",.	c #C5D9BD",
"'.	c #52B727",
").	c #50A72A",
"!.	c #2B9000",
"~.	c #FBFFFA",
"{.	c #D5FFC3",
"].	c #9CDB80",
"^.	c #74C253",
"/.	c #F8FFF5",
"(.	c #98D17F",
"_.	c #E0FFD3",
":.	c #BFFFA3",
"<.	c #90DC70",
"[.	c #36A208",
"}.	c #DEFFD0",
"|.	c #2E9401",
"1.	c #ABE691",
"2.	c #489925",
"3.	c #6ACF3D",
"4.	c #66C03E",
"5.	c #6BCD41",
"6.	c #EAFFE2",
"7.	c #54AC2E",
"8.	c #288600",
"9.	c #A2E485",
"0.	c #83C268",
"a.	c #5ABB31",
"b.	c #E3FFD7",
"c.	c #F1FFEB",
"d.	c #94C37F",
"e.	c #93C07F",
"f.	c #9DE37F",
"g.	c #D9FFC9",
"                                                                ",
"                                                                ",
"                    .   +                                       ",
"                + + @ # .                                       ",
"        . $     + % & * = - - ;                                 ",
"  . > * * * > , ' * ) & . ! * > ~                               ",
"{ * * * * * * * ] ^ / ( _ : * < [                               ",
"} | * * * * * * * * * * * * * > 1 2 3 4                         ",
"  5 * * * * * * * * * * * * * * * * * > 6 7                     ",
"    8 9 * * * * * * * * * * * * * * * * * 0 a                   ",
"        b c d e f g > * * * * * * * * * * * ] h                 ",
"          i   .     j * k * * * * * * * * * * l                 ",
"        . m n o p   q * . r s * * * * * * * * * t + * u v .     ",
"            & * w x y * .   z A B * * * * * * * C D * * . E     ",
"        & * . F G H I I J         K L * * * * * * * * * / M     ",
"        + + + N                     O P Q * * * * * * * * R + . ",
"            + S                     T U * * * * * V W * X Y Z ` ",
"                                    & * * * * * *  .  ..+.      ",
"                                . * @.] * #.$.%.* * &.  *.&     ",
"                                  . =.o * m -.;.* * >.  ,.+     ",
"                                    & '.).!.~.{.* * ].          ",
"                                + g ^.  /.*   O * * (.          ",
"                                  .       * . _.* * :.          ",
"                                          +   <.* [.            ",
"                                            }.|.* 1.            ",
"                                            2.| 3.              ",
"                                          4.| 5.                ",
"                                      6.7.8.9.                  ",
"                                  7 0.9 a.b.                    ",
"                          c.. . d.e.f.g.                        ",
"                                                                ",
"                                                                "};



/************************/
/* GECCO_PLUGIN_INIT	*/
/************************/
/*
 * Register plugin with gecco
 *
 * Returns NULL on failure, a pointer on success.
 */
void *gecco_plugin_init ( command )
	int	command;
	{
	if ( !command )
		/*
		 * command = 0 -- Plugin should 'register' itself.
		 */
		return ( (void *)gecco_plugin_register (
					GECCOUSERMAN_NAME,
					GECCOUSERMAN_VERSION,
					GECCOUSERMAN_FULLNAME,
					GECCOUSERMAN_AUTHORS,
					GECCOUSERMAN_DESCRIPTION,
					dbuttons,
					geccoicon_xpm ) );
	else
		/*
		 * command != 0 -- Plugin should 'cleanup'.
		 */
		/*
		 * 'gusermanshutdown' should return FALSE (0) upon
		 * successful shutdown.
		 */
		if ( !gusermanshutdown ( NULL, guserman ) )
			return ( "Success" );

	/*
	 * If commands fail, return NULL
	 */
	return ( NULL );
	}



/************************/
/* GECCO_PLUGIN_RUN	*/
/************************/
/*
 * Execute plugin
 */
int gecco_plugin_run ( display )
	GtkWidget	*display;
	{

	/*
	 * If display is NULL, then return.
	 */
	if ( display == NULL )
		return ( FALSE );

	/*
	 * If guserman has already been defined, leave; we don't want
	 * more that one copy running at a time.
	 *
	 * This should not be necessary, as gecco takes care that only
	 * one copy can run at a time.
	 */
	if ( guserman != NULL )
		return ( FALSE );


	guserman = geccouserman_new ( display );

	if ( guserman == NULL )	/* For some reason, UserMan can't run! */
		return ( FALSE );

	/*
	 * Make main window insensitive until everything is set up
	 */
	gtk_widget_set_sensitive ( guserman->win, FALSE );

	/*
	 * Make the display visible
	 */
	gtk_widget_show_all ( guserman->win );


	/*
	 * Do your thing here
	 */	

	/*
	 * Make sure display is completely visible before continuing
	 */
	while ( gtk_events_pending () )
		gtk_main_iteration ();

	/*
	 * Done everything; make display sensitive again
	 */
	gtk_widget_set_sensitive ( guserman->win, TRUE );

	return ( TRUE );
	}







/************************/
/* GECCOUSERMAN_NEW	*/
/************************/
static geccouserman_t *geccouserman_new ( display )
	GtkWidget	*display;
	{
	geccouserman_t	*localguserman;
	GtkWidget	*label;
	GtkWidget	*userman;


	/*
	 * Allocate memory for the app
	 */
	localguserman = ( geccouserman_t *) malloc ( sizeof ( geccouserman_t ) );
	if ( localguserman == NULL )
		return NULL;


	/*
	 * Connect to main dialog window
	 */
	localguserman->win = display;
	gnome_dialog_set_close ( GNOME_DIALOG ( localguserman->win ), FALSE );

	/* 
	 * Create the user manager widge
	 */
	if ((userman = userman_new()) == NULL)
		return (NULL);	/* couldn't run UserMan */

	gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(localguserman->win)->vbox),
		userman, TRUE, TRUE, GNOME_PAD/2 );

	/*
	 * Set the 'Close' button to be default
	 */
	gnome_dialog_set_default ( GNOME_DIALOG ( localguserman->win ), 2 );


	/*
	 * Make the 'Apply' button insensitive
	 */
	gnome_dialog_set_sensitive ( GNOME_DIALOG (
				localguserman->win ), 1, FALSE );


	/*
	 * Call 'main_button_press' when a button on the main window is clicked.
	 */
	gtk_signal_connect ( GTK_OBJECT ( localguserman->win ), "clicked",
				GTK_SIGNAL_FUNC ( main_button_press ),
				localguserman );



	/*
	 * Connect 'close' signal to 'gusermanshutdown' function
	 */
	gtk_signal_connect ( GTK_OBJECT ( localguserman->win ), "close",
				GTK_SIGNAL_FUNC ( gusermanshutdown ), localguserman );


	/*
	 * Initialize tooltips
	 */
	localguserman->tips = gtk_tooltips_new ();
	gtk_object_set_data ( GTK_OBJECT ( localguserman->win ),
				"geccouserman", localguserman->tips );



	/*
	 * Create a label, just for kicks
	 */
	label = gtk_label_new ( _(GECCOUSERMAN_DESCRIPTION) );
	gtk_box_pack_start ( GTK_BOX ( GNOME_DIALOG ( localguserman->win)->vbox ),
				label,
				TRUE, TRUE, GNOME_PAD/2 );


	return ( localguserman );
	}



/************************/
/* MAIN_BUTTON_PRESS	*/
/************************/
static int main_button_press ( dialog, button, localguserman )
	GtkWidget	*dialog;
	gint		button;
	geccouserman_t	*localguserman;
	{
	if ( button == 0 )		// 'Ok' button
		gnome_dialog_close ( GNOME_DIALOG ( dialog ) );
	if ( button == 2 )		// 'Cancel' button
		gnome_dialog_close ( GNOME_DIALOG ( dialog ) );
	if ( button == 3 )		// 'Help' button
		help_button ( localguserman );

	return ( TRUE );
	}



/****************/
/* HELP_BUTTON	*/
/****************/
void help_button ( localguserman )
	geccouserman_t	*localguserman;
	{
	GtkWidget	*win;

	win = gnome_about_new ( GECCOUSERMAN_NAME,
				GECCOUSERMAN_VERSION,
				GECCOUSERMAN_COPYRIGHT,
				GECCOUSERMAN_AUTHORS,
				GECCOUSERMAN_DESCRIPTION,
				NULL );
	gnome_dialog_set_parent ( GNOME_DIALOG ( win ),
				GTK_WINDOW ( localguserman->win ) );
	gtk_widget_show ( win );
	}




/****************/
/* SHUTDOWN	*/
/****************/
/*
 * Must return FALSE or plugin won't close
 */
static int gusermanshutdown ( widget, localguserman )
	GtkWidget	*widget;
	geccouserman_t	*localguserman;
	{

	main_quit(NULL);

	if ( localguserman != NULL )
		free ( localguserman );

	guserman = NULL;

	return ( FALSE );
	}


